@@ -1,4 +1,4 @@
-require 'json'
+require 'ffi_yajl'
 require 'chef_zero/endpoints/rest_list_endpoint'
 
 module ChefZero
@@ -7,7 +7,7 @@ module ChefZero
     class DataBagsEndpoint < RestListEndpoint
       def post(request)
         contents = request.body
-        json = JSON.parse(contents, :create_additions => false)
+        json = FFI_Yajl::Parser.parse(contents, :create_additions => false)
         name = identity_keys.map { |k| json[k] }.select { |v| v }.first
         if name.nil?
           error(400, "Must specify #{identity_keys.map { |k| k.inspect }.join(' or ')} in JSON")
