@@ -2,6 +2,9 @@ require 'rack/request'
 
 module ChefZero
   class RestRequest
+
+    ZERO = "0".freeze
+
     def initialize(env, rest_base_prefix = [])
       @env = env
       @rest_base_prefix = rest_base_prefix
@@ -19,7 +22,11 @@ module ChefZero
     end
 
     def api_version
-      @env['HTTP_X_OPS_SERVER_API_VERSION'] || 0
+      @env['HTTP_X_OPS_SERVER_API_VERSION'] || ZERO
+    end
+
+    def api_v0?
+      api_version == ZERO
     end
 
     def requestor
@@ -34,6 +41,10 @@ module ChefZero
       @rest_path ||= rest_base_prefix + env['PATH_INFO'].split('/').select { |part| part != "" }
     end
 
+    def rest_path=(rest_path)
+      @rest_path = rest_path
+    end
+
     def body=(body)
       @body = body
     end
