@@ -40,8 +40,14 @@ require 'chef_zero/endpoints/rest_list_endpoint'
 require 'chef_zero/endpoints/authenticate_user_endpoint'
 require 'chef_zero/endpoints/acls_endpoint'
 require 'chef_zero/endpoints/acl_endpoint'
-require 'chef_zero/endpoints/actors_endpoint'
 require 'chef_zero/endpoints/actor_endpoint'
+require 'chef_zero/endpoints/actors_endpoint'
+require 'chef_zero/endpoints/actor_key_endpoint'
+require 'chef_zero/endpoints/organization_user_key_endpoint'
+require 'chef_zero/endpoints/organization_user_default_key_endpoint'
+require 'chef_zero/endpoints/organization_user_keys_endpoint'
+require 'chef_zero/endpoints/actor_default_key_endpoint'
+require 'chef_zero/endpoints/actor_keys_endpoint'
 require 'chef_zero/endpoints/cookbooks_endpoint'
 require 'chef_zero/endpoints/cookbook_endpoint'
 require 'chef_zero/endpoints/cookbook_version_endpoint'
@@ -105,9 +111,9 @@ module ChefZero
   class Server
 
     DEFAULT_OPTIONS = {
-      :host => '127.0.0.1',
+      :host => ['127.0.0.1'],
       :port => 8889,
-      :log_level => :info,
+      :log_level => :warn,
       :generate_real_keys => true,
       :single_org => 'chef',
       :ssl => false
@@ -136,6 +142,7 @@ module ChefZero
     def port
       if @port
         @port
+      # If options[:port] is not an Array or an Enumerable, it is just an Integer.
       elsif !options[:port].respond_to?(:each)
         options[:port]
       else
@@ -158,10 +165,11 @@ module ChefZero
     #
     def url
       sch = @options[:ssl] ? 'https' : 'http'
-      @url ||= if @options[:host].include?(':')
-                 URI("#{sch}://[#{@options[:host]}]:#{port}").to_s
+      hosts = Array(@options[:host])
+      @url ||= if hosts.first.include?(':')
+                 URI("#{sch}://[#{hosts.first}]:#{port}").to_s
                else
-                 URI("#{sch}://#{@options[:host]}:#{port}").to_s
+                 URI("#{sch}://#{hosts.first}:#{port}").to_s
                end
     end
 
@@ -259,6 +267,18 @@ module ChefZero
     # @return [Thread]
     #   the thread the background process is running in
     #
+    def listen(hosts, port)
+      hosts.each do |host|
+        @server.listen(host, port)
+      end
+      true
+    rescue Errno::EADDRINUSE
+      ChefZero::Log.warn("Port #{port} not available")
+      @server.listeners.each { |l| l.close }
+      @server.listeners.clear
+      false
+    end
+
     def start_background(wait = 5)
       @server = WEBrick::HTTPServer.new(
         :DoNotListen => true,
@@ -274,22 +294,17 @@ module ChefZero
       @server.mount('/', Rack::Handler::WEBrick, app)
 
       # Pick a port
-      if options[:port].respond_to?(:each)
-        options[:port].each do |port|
-          begin
-            @server.listen(options[:host], port)
-            @port = port
-            break
-          rescue Errno::EADDRINUSE
-            ChefZero::Log.info("Port #{port} in use: #{$!}")
-          end
+      # If options[:port] can be an Enumerator, an Array, or an Integer,
+      # we need something that can respond to .each (Enum and Array can already).
+      Array(options[:port]).each do |port|
+        if listen(Array(options[:host]), port)
+          @port = port
+          break
         end
-        if !@port
-          raise Errno::EADDRINUSE, "No port in :port range #{options[:port]} is available"
-        end
-      else
-        @server.listen(options[:host], options[:port])
-        @port = options[:port]
+      end
+      if !@port
+        raise Errno::EADDRINUSE,
+          "No port in :port range #{options[:port]} is available"
       end
 
       # Start the server in the background
@@ -523,7 +538,7 @@ module ChefZero
         [
           [ "/organizations/*/users", ActorsEndpoint.new(self) ],
           [ "/organizations/*/users/*", ActorEndpoint.new(self) ],
-          [ "/organizations/*/authenticate_user", OrganizationAuthenticateUserEndpoint.new(self) ],
+          [ "/organizations/*/authenticate_user", OrganizationAuthenticateUserEndpoint.new(self) ]
         ]
       else
         # EC-only
@@ -537,11 +552,13 @@ module ChefZero
           [ "/users/*/association_requests", UserAssociationRequestsEndpoint.new(self) ],
           [ "/users/*/association_requests/count", UserAssociationRequestsCountEndpoint.new(self) ],
           [ "/users/*/association_requests/*", UserAssociationRequestEndpoint.new(self) ],
+          [ "/users/*/keys", ActorKeysEndpoint.new(self) ],
+          [ "/users/*/keys/default", ActorDefaultKeyEndpoint.new(self) ],
+          [ "/users/*/keys/*", ActorKeyEndpoint.new(self) ],
           [ "/users/*/organizations", UserOrganizationsEndpoint.new(self) ],
           [ "/authenticate_user", AuthenticateUserEndpoint.new(self) ],
           [ "/system_recovery", SystemRecoveryEndpoint.new(self) ],
           [ "/license", LicenseEndpoint.new(self) ],
-
           [ "/organizations", OrganizationsEndpoint.new(self) ],
           [ "/organizations/*", OrganizationEndpoint.new(self) ],
           [ "/organizations/*/_validator_key", OrganizationValidatorKeyEndpoint.new(self) ],
@@ -564,6 +581,12 @@ module ChefZero
         [ "/dummy", DummyEndpoint.new(self) ],
         [ "/organizations/*/clients", ActorsEndpoint.new(self) ],
         [ "/organizations/*/clients/*", ActorEndpoint.new(self) ],
+        [ "/organizations/*/clients/*/keys", ActorKeysEndpoint.new(self) ],
+        [ "/organizations/*/clients/*/keys/default", ActorDefaultKeyEndpoint.new(self) ],
+        [ "/organizations/*/clients/*/keys/*", ActorKeyEndpoint.new(self) ],
+        [ "/organizations/*/users/*/keys", OrganizationUserKeysEndpoint.new(self) ],
+        [ "/organizations/*/users/*/keys/default", OrganizationUserDefaultKeyEndpoint.new(self) ],
+        [ "/organizations/*/users/*/keys/*", OrganizationUserKeyEndpoint.new(self) ],
         [ "/organizations/*/controls", ControlsEndpoint.new(self) ],
         [ "/organizations/*/cookbooks", CookbooksEndpoint.new(self) ],
         [ "/organizations/*/cookbooks/*", CookbookEndpoint.new(self) ],
