@@ -5,6 +5,42 @@ require 'bundler/setup'
 require 'chef_zero/server'
 require 'rspec/core'
 
+# This file runs oc-chef-pedant specs and is invoked by `rake pedant`
+# and other Rake tasks. Run `rake -T` to list tasks.
+#
+# Options for oc-chef-pedant and rspec can be specified via
+# ENV['PEDANT_OPTS'] and ENV['RSPEC_OPTS'], respectively.
+#
+# The log level can be specified via ENV['LOG_LEVEL'].
+#
+# Example:
+#
+#     $ PEDANT_OPTS="--focus-users --skip-keys" \
+#     >   RSPEC_OPTS="--fail-fast --profile 5" \
+#     >   LOG_LEVEL=debug \
+#     >   rake pedant
+#
+
+DEFAULT_SERVER_OPTIONS = {
+  port: 8889,
+  single_org: false,
+}.freeze
+
+DEFAULT_LOG_LEVEL = :warn
+
+def log_level
+  return ENV['LOG_LEVEL'].downcase.to_sym if ENV['LOG_LEVEL']
+  return :debug if ENV['DEBUG']
+  DEFAULT_LOG_LEVEL
+end
+
+def start_chef_server(opts={})
+  opts = DEFAULT_SERVER_OPTIONS.merge(opts)
+  opts[:log_level] = log_level
+
+  ChefZero::Server.new(opts).tap {|server| server.start_background }
+end
+
 def start_cheffs_server(chef_repo_path)
   require 'chef/version'
   require 'chef/config'
@@ -34,37 +70,42 @@ def start_cheffs_server(chef_repo_path)
   data_store.set(%w(organizations pedant-testorg groups admins), '{ "users": [ "pivotal" ] }')
   data_store.set(%w(organizations pedant-testorg groups users), '{ "users": [ "pivotal" ] }')
 
-  server = ChefZero::Server.new(
-    port: 8889,
-    data_store: data_store,
-    single_org: false,
-    #log_level: :debug
-  )
-  server.start_background
-  server
+  start_chef_server(data_store: data_store)
 end
 
-tmpdir = nil
+def pedant_args_from_env
+  args_from_env('PEDANT_OPTS')
+end
+
+def rspec_args_from_env
+  args_from_env('RSPEC_OPTS')
+end
+
+def args_from_env(key)
+  return [] unless ENV[key]
+  ENV[key].split
+end
 
 begin
-  if ENV['FILE_STORE']
-    require 'tmpdir'
-    require 'chef_zero/data_store/raw_file_store'
-    tmpdir = Dir.mktmpdir
-    data_store = ChefZero::DataStore::RawFileStore.new(tmpdir, true)
-    data_store = ChefZero::DataStore::DefaultFacade.new(data_store, false, false)
-    server = ChefZero::Server.new(:port => 8889, :single_org => false, :data_store => data_store)
-    server.start_background
+  tmpdir = nil
+  server =
+    if ENV['FILE_STORE']
+      require 'tmpdir'
+      require 'chef_zero/data_store/raw_file_store'
+      tmpdir = Dir.mktmpdir
+      data_store = ChefZero::DataStore::RawFileStore.new(tmpdir, true)
+      data_store = ChefZero::DataStore::DefaultFacade.new(data_store, false, false)
 
-  elsif ENV['CHEF_FS']
-    require 'tmpdir'
-    tmpdir = Dir.mktmpdir
-    server = start_cheffs_server(tmpdir)
+      start_chef_server(data_store: data_store)
 
-  else
-    server = ChefZero::Server.new(:port => 8889, :single_org => false)#, :log_level => :debug)
-    server.start_background
-  end
+    elsif ENV['CHEF_FS']
+      require 'tmpdir'
+      tmpdir = Dir.mktmpdir
+      start_cheffs_server(tmpdir)
+
+    else
+      start_chef_server
+    end
 
   require 'rspec/core'
   require 'pedant'
@@ -83,6 +124,7 @@ begin
       '--skip-users',
       '--skip-organizations',
       '--skip-multiuser',
+      '--skip-user-keys',
 
       # chef-zero has some non-removable quirks, such as the fact that files
       # with 255-character names cannot be stored in local mode. This is
@@ -98,13 +140,17 @@ begin
   chef_fs_skips << '--skip-cookbook-artifacts'
   chef_fs_skips << '--skip-policies'
 
+  # Multi-keys don't work prior to 12.8
+  unless Gem::Requirement.new(">= 12.8.0").satisfied_by?(Gem::Version.new(Chef::VERSION))
+    chef_fs_skips << '--skip-keys'
+  end
+
   # These things aren't supported by Chef Zero in any mode of operation:
   default_skips = [
     # "the goal is that only authorization, authentication and validation tests
     # are turned off" - @jkeiser
     #
     # ...but we're not there yet
-    '--skip-keys',
 
     # Chef Zero does not intend to support validation the way erchef does.
     '--skip-validation',
@@ -128,7 +174,6 @@ begin
     '--skip-usags',
 
     # Chef 12 features not yet 100% supported by Chef Zero
-    '--skip-api-v1',
 
     # The universe endpoint is unlikely to ever make sense for Chef Zero
     '--skip-universe',
@@ -143,12 +188,10 @@ begin
       default_skips + chef_fs_skips + %w{ --skip-knife }
     end
 
-  Pedant.setup(pedant_args)
+  Pedant.setup(pedant_args + pedant_args_from_env)
 
-  fail_fast = %w()#--fail-fast)
-  #fail_fast = ["--fail-fast"]
-
-  result = RSpec::Core::Runner.run(Pedant.config.rspec_args + fail_fast)
+  rspec_args = Pedant.config.rspec_args + rspec_args_from_env
+  result = RSpec::Core::Runner.run(rspec_args)
 
   server.stop if server.running?
 ensure
